# Copyright (c) 2002-2006 Infrae. All rights reserved.
# See also LICENSE.txt
# $Revision: 1.25 $

from zope.interface import implements

import os
from types import ListType
import random, time
random.seed(time.time())

from Globals import InitializeClass, package_home
from AccessControl import ClassSecurityInfo, getSecurityManager
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
from zExceptions import NotFound,Unauthorized

from Products.SilvaExternalSources.interfaces import IExternalSource
from Products.SilvaExternalSources.CodeSource import CodeSource

from Products.Silva import SilvaPermissions
from Products.Silva.helpers import add_and_edit
from Products.Silva.interfaces import IContainer

from OFS.Image import Image

from Products.Formulator.Form import ZMIForm
from Products.Formulator.XMLToForm import XMLToForm

from zLOG import LOG,INFO

def ustr(x):
    if type(x) == unicode:
        return x
    elif type(x) == str:
        return unicode(x, 'utf-8')
    return str(x)

intervalJS = """
var itemscache%(random_id)s = new Object();
function swapitems%(random_id)s() {
  try { /* see if ajax exists.  If not, turn off interval and raise an alert */
    Ajax;
  } catch (ex) {
    clearInterval(intervalNum%(random_id)s);
    alert('prototype javascript library is not available, cancelling auto-rotate');
    return;
  } 

  var items = %(random_items)s; /* list of items in folder */
  var dt = %(dt)s; /* display title */
  var dcd = %(dcd)s; /* display content description */
  var interval = %(interval)s; /* seconds */
  var object_id = '%(object_id)s'; /* ID of random cs div */
  var cs_url = '%(cs_url)s';
  //get a random item, retrieve it, and replace current item
  var random_item = items[Math.floor(Math.random()*items.length)];
  var replace_content = function(newtext) {
    var obj = $(object_id);
    for (i=obj.childNodes.length-1; i>=0; i--) {
      n = obj.childNodes[i];
      // the node we're interested in will _always_ be the last <div> element
      if (n.nodeType == 1 && n.nodeName.toLowerCase() == 'div') {
	//for some reason, this doesn't work in Firefox2...the nodes are imported,
	//but no styling is associated with them, no images, loaded, etc.
	//newNode = document.importNode(xmlhttp.responseXML.firstChild,true);
	//to_replace = obj.replaceChild(newNode, n);
	//so do this instead (and it's cross-browser compliant, I think
	n.innerHTML = newtext;
	break;
      }
    }
  };

  if (!itemscache%(random_id)s[random_item]) {
    // add query items to cs_url
    cs_url += '/getIntervalItem?dt=' + (dt?true:'') + '&dcd=' + (dcd?true:'') + 
      '&item_path=' + encodeURIComponent(random_item);
    new Ajax.Request(cs_url,
		     {method: "get",
			 onSuccess: function(xmlhttp) {
			              replace_content(xmlhttp.responseText);
				      itemscache%(random_id)s[random_item] = xmlhttp.responseText;
                 		    },
			 onFailure: function(xmlhttp) { clearInterval(intervalNum%(random_id)s); },
			 onException: function(req, exc) { throw exc; } 
		     }
		    );
  } else {
    replace_content(itemscache%(random_id)s[random_item]);
  }
};
intervalNum%(random_id)s = setInterval(swapitems%(random_id)s,%(interval)s*1000);
"""

class RandomContentSource(CodeSource):
    """A Random Content Source
    """

    implements(IExternalSource)
    meta_type = 'Silva Random Content Source'
    security = ClassSecurityInfo()

    def __init__(self, id, title):
        CodeSource.inheritedAttribute('__init__')(self, id, title)
        self._script_id = 'view'
        self._data_encoding = 'utf-8'

    def manage_afterAdd(self, item, container):
        self._set_form()
        self._set_views()

    security.declareProtected(SilvaPermissions.ChangeSilvaAccess,
                                'refresh')
    def refresh(self):
        """reload the form and pt"""
        oids = self.objectIds()
        for i in ['view','view_macros','interval_item']:
            if i in oids:
                self.manage_delObjects([i])
        self._set_form()
        self._set_views()
        return 'refreshed form and pagetemplate'

    def _set_form(self):
        self.parameters = ZMIForm('form', 'Properties Form')
        f = open(os.path.join(
                package_home(globals()), 
                'www', 
                'random_content_form.form'))
        XMLToForm(f.read(), self.parameters)
        f.close()

    def _set_views(self):
        f = open(os.path.join(
                package_home(globals()),
                'www',
                'random_content_view.pt'))
        self._setObject('view', ZopePageTemplate('view', f.read()))
        f.close()

        f = open(os.path.join(
                package_home(globals()),
                'www',
                'random_content_view_macros.pt'))
        self._setObject('view_macros', ZopePageTemplate('view_macros', f.read()))
        f.close()

        f = open(os.path.join(
                package_home(globals()),
                'www',
                'interval_item.pt'))
        self._setObject('interval_item', ZopePageTemplate('interval_item', f.read()))
        f.close()

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                                'getItemsInFolder')
    def getItemsInFolder(self, folder, meta_types, depth=0):
        depth = int(depth)
        if not isinstance(folder,ListType):
            folder = folder.split('/')
        folder = self.restrictedTraverse(folder)
        user = getSecurityManager().getUser()

        items = []
        def _getItemsHelper(container, curdepth):
            for item in container.objectValues():
                if not user.has_permission(SilvaPermissions.View, item):
                    continue
                if item.meta_type in meta_types:
                    items.append(item)
                if IContainer.providedBy(item) and item.is_transparent() and \
                       (depth == -1 or curdepth < depth):
                    LOG('depth',INFO,str(curdepth) + " " + str(depth) + ' ' + str(curdepth < depth))
                    _getItemsHelper(item, curdepth + 1)
        _getItemsHelper(folder, 0)
        LOG('items',INFO,str(items))
        return items

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                                'getRandomItem')
    def getRandomItem(self, items):
        return random.choice(items)

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'getIntervalItem')
    def getIntervalItem(self, item_path, dcd, dt):
        """for the interval switching, retrieve the next item"""
        user = getSecurityManager().getUser()
        item = self.restrictedTraverse(item_path)
        if not user.has_permission(SilvaPermissions.View, item):
            #user doesn't have permissions to it, so raise error
            raise Unauthorized
        self.REQUEST.RESPONSE.setHeader('Content-Type','text/xml; charset=utf-8');
        cachetime = self.getCacheTime()
        self.REQUEST.RESPONSE.setHeader('Cache-Control','max-age='+cachetime)
        return self.interval_item(random_item=item, display_title=dt, display_content_desc=dcd)

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'generateIntervalScript')
    def generateIntervalScript(self, interval, items, display_title, display_content_desc, object_id):
        #hopefully randomize a bit, to generate a random swapitems_RANDOM func
        random_id = str(int(random.random()*time.time()))
        cs_url = self.aq_parent.absolute_url() + '/' + self.id
        return intervalJS%({'random_id':random_id,
                          'random_items':str(['/'.join(i.getPhysicalPath()) for i in items]),
                          'dt':str(display_title).lower(),
                          'dcd':str(display_content_desc).lower(),
                          'interval':interval,
                          'object_id':object_id,
                          'cs_url':cs_url})
        
    

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'render_doc')
    def render_doc(self, doc):
        oldmodel = self.REQUEST.get('model',None)
        self.REQUEST['model'] = doc
        self.REQUEST['suppress_title'] = 0
        ct = ustr(getattr(doc, 'view')())
        self.REQUEST['model'] = oldmodel
        return ct

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                                'to_html')
    def to_html(self, *args, **kwargs):
        """render the news list"""
        oldmodel = self.REQUEST.get('model',None)
        self.REQUEST['model'] = self
        ct = ustr(getattr(self, 'view')(**kwargs))
        self.REQUEST['model'] = oldmodel
        return ct

InitializeClass(RandomContentSource)

manage_addRandomContentSourceForm = PageTemplateFile(
    "www/randomContentSourceAdd", globals(), __name__='manage_addRandomContentSourceForm')

def manage_addRandomContentSource(context, id, title, REQUEST=None):
    """Add a Random Content Source"""
    v = RandomContentSource(id, unicode(title, 'utf-8'))
    context._setObject(id, v)
    add_and_edit(context, id, REQUEST)
    return ''
